Package: spatsoc
Title: Group Animal Relocation Data by Spatial and Temporal
        Relationship and Measure Intragroup Social Dynamics
Version: 0.2.13
Authors@R: c(
    person("Alec L.", "Robitaille", , "robit.alec@gmail.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-4706-1762")),
    person("Quinn", "Webber", role = "aut",
           comment = c(ORCID = "0000-0002-0434-9360")),
    person("Eric", "Vander Wal", role = "aut",
           comment = c(ORCID = "0000-0002-8534-4317"))
  )
Description: Detects spatial and temporal groups in GPS relocations
    (Robitaille et al. (2019) <doi:10.1111/2041-210X.13215>).  It can be
    used to convert GPS relocations to gambit-of-the-group format to build
    proximity-based social networks, and perform data-stream randomization 
    methods suitable for GPS data. Also provides measures of intragroup social 
    dynamics including distance and direction to leaders, centroids and nearest 
    neighbours. 
License: GPL-3 | file LICENSE
URL: https://docs.ropensci.org/spatsoc/,
        https://github.com/ropensci/spatsoc
BugReports: https://github.com/ropensci/spatsoc/issues
Depends: R (>= 3.4)
Imports: adehabitatHR (>= 0.4.21), data.table (>= 1.15.0), igraph, sf,
        lwgeom (>= 0.2.15), CircStats, stats, units (>= 0.8-6), rlang
Suggests: asnipe, knitr, markdown, move2, rmarkdown, s2, sftrack,
        testthat (>= 2.1.0)
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.3.3
SystemRequirements: GDAL (>= 2.0.1), GEOS (>= 3.4.0), PROJ (>= 4.8.0),
        sqlite3
NeedsCompilation: no
Packaged: 2026-04-27 12:47:46 UTC; alecr
Author: Alec L. Robitaille [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-4706-1762>),
  Quinn Webber [aut] (ORCID: <https://orcid.org/0000-0002-0434-9360>),
  Eric Vander Wal [aut] (ORCID: <https://orcid.org/0000-0002-8534-4317>)
Maintainer: Alec L. Robitaille <robit.alec@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-27 13:20:03 UTC
Built: R 4.4.3; ; 2026-05-01 14:16:29 UTC; unix
