# Generated by roxygen2: do not edit by hand

export(build_lines)
export(build_polys)
export(centroid_dyad)
export(centroid_fusion)
export(centroid_group)
export(direction_group)
export(direction_polarization)
export(direction_step)
export(direction_to_centroid)
export(direction_to_leader)
export(distance_to_centroid)
export(distance_to_leader)
export(dyad_id)
export(edge_alignment)
export(edge_delay)
export(edge_direction)
export(edge_dist)
export(edge_nn)
export(edge_zones)
export(fusion_id)
export(get_gbi)
export(get_geometry)
export(group_lines)
export(group_polys)
export(group_pts)
export(group_times)
export(leader_direction_group)
export(leader_edge_delay)
export(randomizations)
import(data.table)
